"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Throttle = void 0;
/**
 * Simple throttle to avoid sending too many concurrent request
 */
class Throttle {
    /**
     * @param workers Number of concurrent request we tolerate.
     */
    constructor(workers) {
        this.workers = workers;
        this.queue = [];
        this.throttle = this.throttle.bind(this);
        this.runQueue = this.runQueue.bind(this);
        this.run = this.run.bind(this);
    }
    throttle(callback) {
        // If we still have workers avaliable, run the job right away
        if (this.workers > 0) {
            return new Promise((resolve) => this.run(resolve, callback));
        }
        // Otherwise queue
        return new Promise((resolve) => {
            this.queue.push([resolve, callback]);
        });
    }
    runQueue() {
        const job = this.queue.shift();
        if (!job) {
            return;
        }
        this.run(...job);
    }
    run(resolve, callback) {
        this.workers--;
        callback().then((response) => {
            this.workers++;
            this.runQueue();
            resolve(response);
        });
    }
}
exports.Throttle = Throttle;
