"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TravisClient = void 0;
// @ts-ignore
const Throttle_1 = require("./Throttle");
const typed_rest_client_1 = require("typed-rest-client");
class TravisClient {
    constructor(token, domain = 'com', log = (msg) => { }) {
        this.token = token;
        this.log = log;
        this.throttler = new Throttle_1.Throttle(20);
        this.client = new typed_rest_client_1.RestClient('fetch-traviss-data', `https://api.travis-ci.${domain}`);
        this.get = this.get.bind(this);
        this.getAll = this.getAll.bind(this);
        this.getBranches = this.getBranches.bind(this);
        this.getLastBuildForBranch = this.getLastBuildForBranch.bind(this);
    }
    args() {
        return {
            "Content-Type": "application/json",
            'Travis-API-Version': 3,
            'User-Agent': 'API Explorer',
            Authorization: `token ${this.token}`
        };
    }
    get(endpoint, params = {}) {
        return this.throttler.throttle(() => {
            return new Promise((resolve) => {
                const options = {
                    queryParameters: { params },
                    additionalHeaders: this.args()
                };
                this.log(`/${endpoint} ${JSON.stringify(params)}`);
                this.client.get(`/${endpoint}`, options).then((response) => {
                    this.log(`Status code: ${response.statusCode}`);
                    resolve(response.result);
                });
            });
        });
    }
    getAll(endpoint, key, parameters = {}) {
        return this.get(endpoint, parameters).then(data => {
            if (!data || !data['@pagination']) {
                return {};
            }
            const { is_last, next } = data['@pagination'];
            if (is_last) {
                return data;
            }
            else {
                const pageData = data[key];
                const { limit, offset } = next;
                return this.getAll(endpoint, key, Object.assign(Object.assign({}, parameters), { limit, offset }))
                    .then(followupData => (Object.assign(Object.assign({}, data), { [key]: [...pageData, ...followupData[key]] })));
            }
        });
    }
    getFirst(endpoint, key, parameters = {}) {
        return this.get(endpoint, parameters).then(data => {
            if (!data[key] || !data[key][0]) {
                return {};
            }
            return data[key][0];
        });
    }
    getRepos(org) {
        return this.getAll(`owner/github/${org}/repos`, 'repositories', { private: false, active: true })
            .then(data => data['repositories'])
            .catch((error) => {
            console.error(error);
            return [];
        });
    }
    getBranches(repo) {
        return this.getAll(`repo/${encodeURIComponent(repo)}/branches`, 'branches', { exists_on_github: true })
            .then(data => data.branches)
            .then((branches) => {
            return Promise.all(branches.map((branch) => this.getLastBuildForBranch(repo, branch.name).then(last_build => (Object.assign(Object.assign({}, branch), { last_build })))));
        })
            .catch((error) => {
            console.error(error);
            return false;
        });
    }
    getLastBuildForBranch(repo, branch) {
        return this.getFirst(`repo/${encodeURIComponent(repo)}/builds`, 'builds', { 'branch.name': branch, 'event_type': 'push,api,cron', 'sort_by': 'number:desc', 'limit': 1 }).then(data => {
            return data;
        })
            .catch((error) => {
            console.error(error);
            return [];
        });
    }
}
exports.TravisClient = TravisClient;
